/**
 * 
 */
package org.gcube.portlets.user.csvimportwizard.ws.client;

import org.gcube.portlets.widgets.lighttree.client.ItemType;
import org.gcube.portlets.widgets.lighttree.client.event.PopupEvent;
import org.gcube.portlets.widgets.lighttree.client.event.PopupHandler;
import org.gcube.portlets.widgets.lighttree.client.load.WorkspaceLightTreeLoadPopup;

import com.extjs.gxt.ui.client.GXT;
import com.extjs.gxt.ui.client.Style.Orientation;
import com.extjs.gxt.ui.client.event.ButtonEvent;
import com.extjs.gxt.ui.client.event.SelectionListener;
import com.extjs.gxt.ui.client.widget.button.Button;
import com.extjs.gxt.ui.client.widget.form.AdapterField;
import com.extjs.gxt.ui.client.widget.form.Field;
import com.extjs.gxt.ui.client.widget.form.MultiField;
import com.extjs.gxt.ui.client.widget.form.TextField;

/**
 * @author "Federico De Faveri defaveri@isti.cnr.it"
 *
 */
public class WorkspaceUploadField extends MultiField<String> {

	protected TextField<String> fileUploadField;
	protected Button browseButton;
	protected String workspaceItemId;

	public WorkspaceUploadField(String fieldLabel)
	{
		super();
		setFieldLabel(fieldLabel);

		fileUploadField = new TextField<String>();
		fileUploadField.setReadOnly(true);
		fileUploadField.setAllowBlank(false);
		fileUploadField.setToolTip("Select the csv file to import from your workspace");
		fileUploadField.setEmptyText("a CSV file...");

		add(fileUploadField);

		browseButton = new Button("Browse");
		browseButton.addSelectionListener(new SelectionListener<ButtonEvent>() {
			
			@Override
			public void componentSelected(ButtonEvent ce) {
				showSelection();
			}
		});
		add(new AdapterField(browseButton));
		
		setSpacing(3);
		setResizeFields(false);
	}
	
	protected void showSelection() {
		WorkspaceLightTreeLoadPopup loadPopup = new WorkspaceLightTreeLoadPopup("Select a CSV file", false, true);
		loadPopup.setShowableTypes(ItemType.EXTERNAL_FILE);
		loadPopup.setSelectableTypes(ItemType.EXTERNAL_FILE);
		loadPopup.setAllowedMimeTypes("text/csv", "application/zip", "application/x-zip", "application/x-zip-compressed", "application/octet-stream", "application/x-compress", "application/x-compressed", "multipart/x-zip");
		
		
		loadPopup.addPopupHandler(new PopupHandler() {
			
			@Override
			public void onPopup(PopupEvent event) {
				if (!event.isCanceled()) {
					fileUploadField.setValue(event.getSelectedItem().getName());
					workspaceItemId = event.getSelectedItem().getId();
				}
			}
		});
		
		loadPopup.center();
		loadPopup.show();
	}

	/**
	 * @return the fileUploadField
	 */
	public TextField<String> getFileUploadField() {
		return fileUploadField;
	}

	/**
	 * @return the browseButton
	 */
	public Button getBrowseButton() {
		return browseButton;
	}

	/**
	 * @return the workspaceItemId
	 */
	public String getWorkspaceItemId() {
		return workspaceItemId;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void onResize(int width, int height) {
		super.onResize(width, height);

		if (orientation == Orientation.HORIZONTAL) {
			if (!GXT.isBorderBox) {
				width -= ((fields.size() - 1) * spacing);
			}
			int fileUploadFieldWidth = width - browseButton.getWidth() - spacing;
			fileUploadField.setWidth(fileUploadFieldWidth);
		} else {
			for (Field<?> f : fields) {
				f.setWidth(width);
			}
		}

		if (GXT.isIE) {
			el().repaint();
		}
	}
}
